package grobs;

import java.awt.Color;
import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.Area ;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

import java.util.*;

import turtleWorld.Assert;

public abstract class Grob extends Observable {
	protected GrobArena arena = null ;
    protected Color outlineColor = Color.BLACK ;
    protected Color fillColor = null ;
    protected int zIndex = 0 ;
    protected boolean isDraggable = true ;
    protected boolean isDropTarget = false ;
	private boolean collidable = false ;

    void setArena( GrobArena arena ) {
        this.arena = arena ; }

    public GrobArena getArena() {
        return this.arena ; }

    public abstract void paint( Graphics2D graphics ) ;

    public abstract Shape getShape() ;
    
    public abstract void moveBy( double dx, double dy ) ;
    
    public void setOutlineColor( Color c ) {
    	outlineColor = c ;
    }
    
    public void setFillColor( Color c ) {
    	fillColor = c ;
    }
    
    public void setCollidable( boolean collidable ) { this.collidable = collidable ; }
    
    public boolean isCollidable() { return this.collidable  ; }
    
    public void setZIndex( int z ) { this.zIndex = z ; }
    
    public int getZIndex() { return this.zIndex ; }
    
    public void setDraggable( boolean d ) { this.isDraggable = d ; }
    
    public boolean isDraggable() { return this.isDraggable ; }
    
    public void setDropTarget( boolean d ) { this.isDropTarget = d ; }
    
    public boolean isDropTarget() { return this.isDropTarget ; }

    public boolean intersects( Grob grob ) {
    	return this.zIndex == grob.zIndex && intersects( grob.getShape() ) ;
    }

    public boolean intersects( Shape shape1 ) {
        Shape shape0 = getShape();
        //System.out.println( "shape0="+shape0);
        //System.out.println( "shape1="+shape1);
        Area area0 = new Area( shape0 ) ;
        Area area1 = new Area( shape1 ) ;
        //System.out.println( "area0="+area0);
        //System.out.println( "area1="+area1);
        area0.intersect( area1 ) ;
        //System.out.println( "area0="+area0);
        boolean result = ! area0.isEmpty() ;
        //if( result ) System.out.println( "INTERSECTION") ;
        return result ;
    }
    
    public Line2D lineOfIntersection(Grob grob) {
    	// TODO finish up collision response.
    	// Limited for now to ellipses and rectangles.
    	Shape shape0 = this.getShape() ;
    	Ellipse2D ellipse0 = null ;
    	Rectangle2D rectangle0 = null ;
    	if( shape0 instanceof Ellipse2D ) ellipse0 = (Ellipse2D) shape0 ;
    	else if( shape0 instanceof Rectangle2D ) rectangle0 = (Rectangle2D) shape0 ;
    	else Assert.assertTrue(false, "Only rectangles and ellipses are supported for now") ;
    	
    	Shape shape1 = grob.getShape() ;
    	Ellipse2D ellipse1 = null ;
    	Rectangle2D rectangle1 = null ;
    	if( shape1 instanceof Ellipse2D ) ellipse1 = (Ellipse2D) shape1 ;
    	else if( shape1 instanceof Rectangle2D ) rectangle1 = (Rectangle2D) shape1 ;
    	else Assert.assertTrue(false, "Only rectangles and ellipses are supported for now") ;
    	
    	if( ellipse0 != null && ellipse1 != null ) return lineOfIntersection( ellipse0, ellipse1) ;
    	else if( ellipse0 != null && rectangle1 != null) return lineOfIntersection( ellipse0, rectangle1) ;
    	else if( rectangle0 != null && ellipse1 != null ) return lineOfIntersection( ellipse1, rectangle0 ) ;
    	else if( rectangle0 != null && rectangle1 != null ) return lineOfIntersection( rectangle0, rectangle1) ;
    	else { Assert.assertTrue(false, "Not reachable") ; return null ; }
    }
    
    private Line2D lineOfIntersection( Ellipse2D ellipse0, Ellipse2D ellipse1) {
    	// Crude approximation. Take a line L through the two centres.
    	// Find a point P on that line that is closer to the smaller ellipse.
    	// That passes through a point that is on that line
    	double xc0 = ellipse0.getCenterX() ;
    	double yc0 = ellipse0.getCenterY() ;
    	double xc1 = ellipse1.getCenterX() ;
    	double yc1 = ellipse1.getCenterY() ;
    	double r0 = Math.sqrt(ellipse0.getHeight()*ellipse0.getHeight() + ellipse0.getWidth()*ellipse0.getWidth()) ;
    	double r1 = Math.sqrt(ellipse1.getHeight()*ellipse1.getHeight() + ellipse1.getWidth()*ellipse1.getWidth()) ;
    	double a = r0 / (r0+r1), b = r1 / (r0+r1) ;
    	return null ;
    }
    
    private Line2D lineOfIntersection( Ellipse2D ellipse0, Rectangle2D rect1) {
    	return null ;
    }
    
    private Line2D lineOfIntersection( Rectangle2D rect0, Rectangle2D rect1) {
    	return null ;
    }
}