package turtleWorld;

import java.awt.* ;
import java.awt.event.* ;
import java.lang.reflect.* ;
import javax.swing.* ;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.*;

import grobs.* ;

public class MainFrame extends JFrame {
    private static final long serialVersionUID = -6412571791124100909L;
	private JTabbedPane tabbedPane;
    
    //Construct the frame
    public MainFrame() {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {

        	this.setTitle("Turtle World");
        	JMenu fileMenu = new JMenu("File");
            JMenuItem exitMenuItem = new JMenuItem("Exit");
            exitMenuItem.addActionListener(new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                	System.exit(0);
                }
            });
            JMenu helpMenu = new JMenu("Help");
            JMenuItem aboutMenuItem = new JMenuItem("About");
            aboutMenuItem.addActionListener(new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                	JOptionPane.showMessageDialog(MainFrame.this,
                			"TurtleWorld\n"
                			+"(c) Theodore Norvell\n"
                			+"Licence is granted to modify and redistribute\n"
                			+"this program in creative ways for fun and education.");
                }
            });
            JMenuBar menuBar = new JMenuBar();
            fileMenu.add(exitMenuItem);
            helpMenu.add(aboutMenuItem);
            menuBar.add(fileMenu);
            menuBar.add(helpMenu);
            this.setJMenuBar(menuBar);
            
            this.tabbedPane = new JTabbedPane() ;
            this.getContentPane().add( tabbedPane) ;
        	

            ControllerFactory factory;
            // The following lines add new tabs to the window.
            // Normally each tab should be associated with a controller
            // of a distinct class. This lets you program multiple turtle
            // worlds. Each turtle world has its own arena, log, buttons,
            // turtle(s), and controller.  To add a new tab, create a new subclass
            // of Controller and then add two more commands to the following list.
            // The first command makes a "factory object" that creates the controller.
            // The second command calls method initializeOneTab, which creates the
            // tab and all the widgets that go on the tab and which uses the factory
            // object to create the controller.
            {
            	factory = new ControllerFactory(){
            		@Override public Controller make(GrobArena arena, Log log, ThreadCounter threadCounter) {
            			return new TurtleController0( arena, log, threadCounter ) ;
            		}} ;
            	initializeOneTab( tabbedPane, factory );

            	factory = new ControllerFactory(){
            			@Override public Controller make(GrobArena arena, Log log, ThreadCounter threadCounter) {
            				return new TurtleController1( arena, log, threadCounter ) ;
            			}} ;
            	initializeOneTab( tabbedPane, factory );

            	factory = new ControllerFactory(){
            			@Override public Controller make(GrobArena arena, Log log, ThreadCounter threadCounter) {
            				return new SkyWriter( arena, log, threadCounter ) ;
            			}} ;
            	initializeOneTab( tabbedPane, factory );

            	factory = new ControllerFactory(){
            			@Override public Controller make(GrobArena arena, Log log, ThreadCounter threadCounter) {
            				return new HanoiController( arena, log, threadCounter ) ;
            			}} ;
            	initializeOneTab( tabbedPane, factory );

            	factory = new ControllerFactory(){
            			@Override public Controller make(GrobArena arena, Log log, ThreadCounter threadCounter) {
            				return new RadarTagController( arena, log, threadCounter ) ;
            			}} ;
            	initializeOneTab( tabbedPane, factory );
            }
        	// The following lines help direct keyboard input to the controller
        	// of the currently selected tab.
        	ChangeListener cl = new ChangeListener() {
				@Override public void stateChanged(ChangeEvent e) {
					JComponent c = (JComponent) tabbedPane.getSelectedComponent() ;
					if( c != null ) c.requestFocusInWindow() ;
				} } ;
            this.tabbedPane.addChangeListener( cl) ;
			cl.stateChanged(null) ; // put the focus on the first tab.
        	
            setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE ) ;
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    private interface ControllerFactory {
    	public Controller make( GrobArena arena, Log log, ThreadCounter threadCounter ) ;
    }

    /** Initialize a tab on a tabbed pane to hold all
     * the GUI components associated with one Controller.
     * This routine is also responsible for creating the Controller and all
     * its associated components such as its arena, log, status line, key listener.
     * @param tabbedPane
     * @param factory
     * @throws Exception
     */
    private void initializeOneTab(JTabbedPane tabbedPane, ControllerFactory factory) throws Exception  {
    	JLabel statusBar = new JLabel();
    	JPanel buttonBar = new JPanel();
    	final JTextPane logComponent = new JTextPane();
        JSplitPane jSplitPane = new JSplitPane();
        JScrollPane arenaScrollPane = new JScrollPane();
        JScrollPane logScrollPane = new JScrollPane();
        ThreadCounter threadCounter = new ThreadCounter(statusBar) ;
        
        
    	GrobArena arena = new GrobArena();
        Log log = new Log() {

            public void print( String str ) {
                Document doc = logComponent.getDocument() ;
                try{
                    doc.insertString(doc.getLength(), str, null);
                    logComponent.setCaretPosition( doc.getLength() ); }
                catch( BadLocationException e) {
                    Assert.assertTrue( false, "Unreachable reached" ) ; } }

            public void clear( ) {
                Document doc = logComponent.getDocument() ;
                try{
                    doc.remove(0, doc.getLength() );
                    logComponent.setCaretPosition( doc.getLength() ); }
                catch( BadLocationException e) {
                    Assert.assertTrue( false, "Unreachable reached" ) ; } }
        } ;
        
        //TODO. It is a problem to create the controller before laying everything out. It means that
        // the arena has 0 width and height at construction time. This means that adding Grobs
        // in the constructor can be a problem.
        Controller controller = factory.make( arena, log, threadCounter ) ;
        arena.setListener( new ArenaEventListener(controller) ) ;
        
        
        JPanel panel = new JPanel() ;
       	panel.setLayout(new BorderLayout());
        
        // Here is GUI hierarchy:
        //  tabbedPane -- panel -- buttonBar
        //                      -- jSplitPane -- arenaScrollPane -- arena
        //                                    -- logScrollPane -- log
        //                      -- statusBar
        //             -- other panels ...
        
    	tabbedPane.addTab(controller.controllerName(), panel) ;
        statusBar.setText("Ready.");
        logComponent.setBackground(Color.lightGray);
        logComponent.setFocusable(false);
        jSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
        jSplitPane.setResizeWeight(0.5);
        //contentPane.setMinimumSize(new Dimension(1000, 1000));
        panel.add(statusBar, BorderLayout.SOUTH);
        panel.add(buttonBar, BorderLayout.NORTH);
        panel.add(jSplitPane, BorderLayout.CENTER);
        jSplitPane.add(arenaScrollPane, JSplitPane.TOP);
        jSplitPane.add(logScrollPane, JSplitPane.BOTTOM);
        arenaScrollPane.setViewportView(arena);
        logScrollPane.setViewportView(logComponent);
        jSplitPane.setDividerLocation(300);
        logComponent.setVisible(true);
        arena.setVisible( true );
        
        createButtonBar( buttonBar, controller ) ;
        KeyListener keyListener = new Keys( controller ) ;
        panel.addKeyListener( keyListener ) ;
        panel.setFocusable( true ) ;


        pack() ;
        this.setSize(new Dimension(800, 600));
        validate();
        startInitialThreads(threadCounter, buttonBar, controller) ;
        addWindowListener(new WindowAdapter() {
            public void windowActivated(WindowEvent e) {
                /* MainFrame.this.requestFocusInWindow(); */ }
        });
    }
    
    private class ArenaEventListener implements ArenaListener {

    	private Controller controller;
    	
    	ArenaEventListener( Controller controller ) {
    		this.controller = controller ;
    	}

		@Override
		public void mousePressed(Grob grob) {
    		fire("onMousePressed", new Class[]{Grob.class}, new Object[] {grob});
		}

		@Override
		public void mouseReleased(Grob grob) {
    		fire("onMouseReleased", new Class[]{Grob.class}, new Object[] {grob});
			
		}

		@Override
		public void startDrag(Grob draggedGrob) {
    		fire("onStartDrag", new Class[]{Grob.class}, new Object[] {draggedGrob});
			
		}

		@Override
		public void endDrag(Grob draggedGrob, Grob target) {
			fire("onEndDrag", new Class[]{Grob.class, Grob.class}, new Object[] {draggedGrob, target});
			
		}

		private void fire(String name, Class<?>[] parameterTypes, Object[] arguments ) {
			try {
    			Method method = controller.getClass().getMethod(name,parameterTypes) ;
    			Runnable runnable = new Invoker( name, method, controller, arguments ) ;
    			Thread thread = new Thread( runnable ) ;
    			controller.threadCounter().increment(name) ;
    			thread.start() ;  }
    		catch( NoSuchMethodException e ) {
    			System.err.println("Couldn't find method "+name); }
    		catch( SecurityException e ) {
    			System.err.println("Security exception while dealing with arena event"); }
		}
    	
    }

    private class Keys implements KeyListener {
    	private Controller controller;
		Keys( Controller controller ) {
    		this.controller = controller ;
    	}
        public void keyPressed( KeyEvent event ) {
        	//System.out.println("KeyEvent:" + event ) ;
        	int kc = event.getKeyCode() ;
        	String name ;
        	if( '0' <= kc && kc <= '9' ) {
        		name = "digit"+(kc-(int)'0') ; }
        	else if( 'A' <= kc && kc <= 'Z' ) {
        		name = (""+(char)(kc)).toLowerCase() ; }
        	else {
        		switch( kc ) {
        		case KeyEvent.VK_LEFT : name = "left" ; break ;
        		case KeyEvent.VK_RIGHT : name = "right" ; break ;
        		case KeyEvent.VK_UP : name = "up" ; break ;
        		case KeyEvent.VK_DOWN : name = "down" ; break ;
        		case KeyEvent.VK_HOME : name = "home" ; break ;
        		case KeyEvent.VK_END : name = "end" ; break ;
        		case KeyEvent.VK_PAGE_DOWN : name = "pageDown" ; break ;
        		case KeyEvent.VK_PAGE_UP : name = "pageUp" ; break ;
        		case KeyEvent.VK_ESCAPE : name = "escape" ; break ;
        		default: name = "" ;
        		} }
        	//System.out.println("Name:" + name ) ;
        	if( name.length() > 0 ) {
        		try {
        			Method method = controller.getClass().getMethod(
        					name,
        					new Class[]{} ) ;

        			Runnable runnable = new Invoker( name, method, controller ) ;
        			Thread thread = new Thread( runnable ) ;
        			controller.threadCounter().increment(name) ;
        			thread.start() ;  }
        		catch( NoSuchMethodException e ) {
        			System.err.println("Couldn't find method "+name); }
        		catch( SecurityException e ) {
        			System.err.println("Security exception while dealing with keypress"); } } }

        public void keyReleased( KeyEvent e ) {}

        public void keyTyped( KeyEvent e ) {}
    }

    private void startInitialThreads( ThreadCounter threadCounter,  JPanel buttonBar, Controller controller ) {

        String[] startUp = controller.startUp() ;
        for( int i=0 ; i<startUp.length ; ++i ) {
            String name = startUp[i] ;
            try {
                Method method = controller.getClass().getMethod(
                                       name,
                                       new Class[]{} ) ;

                Runnable runnable = new Invoker( name, method, controller ) ;
                Thread thread = new Thread( runnable ) ;
                threadCounter.increment(name) ;
                thread.start() ;  }
            catch( NoSuchMethodException e ) {
                System.err.println("Couldn't find method "+name+" for start up"); }
            catch( SecurityException e ) {
                System.err.println("Security exception while doing start up"); } }
        buttonBar.revalidate();
    }

    private void createButtonBar( JPanel buttonBar, Controller controller ) {
        buttonBar.setLayout( new FlowLayout() );
        buttonBar.setMinimumSize( new Dimension(100, 100) );
        buttonBar.setPreferredSize( new Dimension(600, 100) );

        String[] buttons = controller.buttons() ;
        for( int i=0 ; i<buttons.length ; ++i ) {
            String name = buttons[i];
            try {
                Method method = controller.getClass().getMethod(
                                       name,
                                       new Class[]{} ) ;
                JButton button = new JButton(buttons[i]) ;
                button.setFocusable(false);
                buttonBar.add( button ) ;
                Runnable runnable = new ButtonInvoker( name, button, method, controller ) ;
                addConcurrentAction( controller, name, button, runnable ) ; }
            catch( NoSuchMethodException e ) {
                System.err.println("Couldn't find method "+name+" to build a button"); }
            catch( SecurityException e ) {
                System.err.println("Security exception while building button"); } }

        buttons = controller.checkBoxes() ;
        for( int i=0 ; i<buttons.length ; ++i ) {
            String name = buttons[i];
            String methodName = name + "On" ;
            try {
                Method onMethod = controller.getClass().getMethod(
                                       methodName,
                                       new Class[]{} ) ;
                methodName = name+"Off" ;
                Method offMethod = controller.getClass().getMethod(
                                       methodName,
                                       new Class[]{} ) ;

                JToggleButton button = new JCheckBox(buttons[i]) ;
                button.setFocusable(false);
                buttonBar.add( button ) ;
                Runnable runnable = new ToggleInvoker( name, button, onMethod, offMethod, controller ) ;
                addConcurrentAction( controller, name, button, runnable ) ; }
            catch( NoSuchMethodException e ) {
                System.err.println("Couldn't find method "+methodName+" to build a button"); }
            catch( SecurityException e ) {
                System.err.println("Security exception while building button"); } }
        buttonBar.revalidate();
    }

    private class Invoker implements Runnable {
    	private Controller controller;
    	
		private String name ;
        private Method method ;
        private Object[] arguments ;
        
        Invoker( String name, Method method, Controller controller ) {
        	this( name, method, controller, new Object[]{} ) ;
        }

        Invoker( String name, Method method, Controller controller, Object[] arguments ) {
            this.name = name ;
            this.method = method ;
            this.controller = controller ;
            this.arguments = arguments ; }

        public void run() {
            try {
                controller.enter() ;
                try { method.invoke( controller, arguments ) ; }
                finally { controller.leave() ; } }
            catch( IllegalAccessException e ) {
                System.err.println("IllegalAccessException while invoking method" );
                e.printStackTrace(System.err); }
            catch( IllegalArgumentException e )  {
                System.err.println("IllegalArgumentException while invoking method" );
                e.printStackTrace(System.err); }
            catch( InvocationTargetException e )  {
                System.err.println("InvocationTargetException while invoking method" );
                System.err.println("Original exception is");
                e.getTargetException().printStackTrace(System.err); }
            finally {
                controller.threadCounter().decrement(name) ; } }
    }

    private class ButtonInvoker implements Runnable {
    	private Controller controller;
    	String name ;
        private Method method ;
        private JButton button ;

        ButtonInvoker( String name, JButton button, Method method, Controller controller ) {
            this.name = name ;
            this.method = method ;
            this.button = button ; 
            this.controller = controller ; }

        public void run() {
            button.setEnabled( false );
            try {
                controller.enter() ;
                try {method.invoke( controller, new Object[]{} ) ;}
                finally { controller.leave() ; } }
            catch( IllegalAccessException e ) {
                System.err.println("IllegalAccessException while invoking method" );
                e.printStackTrace(System.err); }
            catch( IllegalArgumentException e )  {
                System.err.println("IllegalArgumentException while invoking method" );
                e.printStackTrace(System.err); }
            catch( InvocationTargetException e )  {
                System.err.println("InvocationTargetException while invoking method" );
                System.err.println("Original exception is");
                e.getTargetException().printStackTrace(System.err); }
            finally {
                button.setEnabled(true);
                controller.threadCounter().decrement(name) ; } }
    }


    private class ToggleInvoker implements Runnable {
    	private Controller controller;
        private String name ;
        private Method onMethod ;
        private Method offMethod ;
        private JToggleButton button ;

        ToggleInvoker( String name, JToggleButton button, Method onMethod, Method offMethod, Controller controller ) {
            this.name = name ;
            this.onMethod = onMethod ;
            this.offMethod = offMethod ;
            this.button = button ;
            this.controller = controller ; }

        public void run() {
            try {
                if( button.isSelected() ) {
                    controller.enter() ;
                    try {onMethod.invoke( controller, new Object[]{} ) ; }
                    finally { controller.leave() ; } }
                else {
                    controller.enter() ;
                    try{ offMethod.invoke( controller, new Object[]{} ) ;}
                    finally { controller.leave() ; } } }
            catch( IllegalAccessException e ) {
                System.err.println("IllegalAccessException while invoking method" );
                e.printStackTrace(System.err); }
            catch( IllegalArgumentException e )  {
                System.err.println("IllegalArgumentException while invoking method" );
                e.printStackTrace(System.err); }
            catch( InvocationTargetException e )  {
                System.err.println("InvocationTargetException while invoking method" );
                System.err.println("Original exception is");
                e.getTargetException().printStackTrace(System.err); }
            finally {
                controller.threadCounter().decrement(name) ; } }
    }

    private void addConcurrentAction( final Controller controller, final String name, AbstractButton button, final Runnable runnable ) {
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Thread thread = new Thread( runnable ) ;
                controller.threadCounter().increment(name) ;
                thread.start() ;
            }
        });
    }
}