package turtleWorld;

import grobs.GrobArena;
import grobs.ShapeGrob;
import grobs.Turtle;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;

public class RadarTagController extends Controller {
	
	private Turtle alice = new Turtle(Color.red) ;
	
	private Turtle bob = new Turtle(Color.blue) ;
	
	private ShapeGrob r0 = new ShapeGrob( new Rectangle( 0, 0, 50, 30) ) ;

	// The constructor
	public RadarTagController( GrobArena arena, Log log, ThreadCounter threadCounter ) {
		super( arena, log, threadCounter ) ;
		arena.add( alice ) ;
		arena.add( bob ) ;
		arena.add( r0 ) ;
	}

	public String controllerName() {
		return "RadarTag" ;
	}

	@Override
	public String[] buttons() {
		return new String[] {
				"start", "stop" } ; }

	@Override
	public String[] checkBoxes() {
		return new String[] { } ; }

	@Override
	public String[] startUp() {
		return new String[] {
		"reset" } ; }

	public void reset() {
		alice.reset();
		alice.clearTrail() ;
		alice.setTrailLength( Double.POSITIVE_INFINITY ) ;
		bob.reset();
		bob.clearTrail() ;
		bob.setTrailLength( Double.POSITIVE_INFINITY ) ;
		double x = arena.getWidth()  ;
		double y = arena.getHeight()  ;
		alice.setPosition( x-20, y/2 ) ;
		bob.setPosition( 20, y/2 ) ;
		alice.setOrientation( 180 );
		bob.setOrientation( 0 ) ;
		r0.setPosition( x/2, 0 ) ;
		r0.setCollidable( true );
	}
	
	private boolean running = false ;

	public void start() {
		reset() ;
		alice.setSpeed( 10 ) ; 
		bob.setSpeed( 10 ) ; 
		r0.setVelocity( 0, 20 ) ;
		running = true ;
		int count = 0 ;
		int aliceScore = 0 ;
		int bobScore = 0 ;
		while( running ) {
			if( alice.checkRadarFor( bob ) ) {
				alice.setRadarRange( 0 ) ; 
				aliceScore = aliceScore + 1 ;
				log.println( "Alice scores! The score is Alice " +aliceScore+ " Bob " +bobScore ) ;
			}
			if( bob.checkRadarFor( alice ) ) {
				bob.setRadarRange( 0 ) ; 
				bobScore = bobScore + 1 ;
				log.println( "Bob scores!   The score is Alice " +aliceScore+ " Bob " +bobScore ) ;
			}
			pause( 0.1 ) ;
			count = count + 1 ;
			if( count == 30 ) {
				alice.setRadarRange( 40 ) ;
				bob.setRadarRange( 0 ) ;
			}
			if( count == 60 ) {
				alice.setRadarRange( 0 ) ;
				bob.setRadarRange( 40 ) ;
				count = 0 ;
			}
		}
	}

	public void stop() {
		running = false ;
		alice.setSpeed( 0 ) ; 
		bob.setSpeed( 0 ) ; 
	}
	
	private void turn( Turtle t, double deflection ) {
		double a = t.getOrientation() ;
		t.setOrientation( a + deflection ) ;
	}
	
	private void faster( Turtle t, double amount ) {
		double v = t.getSpeed() ;
		t.setSpeed( v + amount ) ;
	}
	
	public void a() {
		turn( alice, -30 ) ;
	}
	
	public void d() {
		turn( alice, +30 ) ;
	}
	
	public void w() {
		faster( alice, +10 ) ;
	}
	
	public void s() {
		faster( alice, -10 ) ;
	}
	
	public void j() {
		turn( bob, -30 ) ;
	}
	
	public void l() {
		turn( bob, +30 ) ;
	}
	
	public void i() {
		faster( bob, +10 ) ;
	}
	
	public void k() {
		faster( bob, -10 ) ;
	}
}